
package matlip;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.color.ColorSpace;
import java.awt.image.BufferedImage;
import java.awt.image.ColorConvertOp;
import java.awt.image.ColorModel;
import java.awt.image.ConvolveOp;
import java.awt.image.Kernel;
import java.awt.image.WritableRaster;
import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.RuntimeException;
import java.util.Hashtable;
import javax.imageio.ImageIO;
import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 *
 * @author hendry
 */
public class Matlip {
   enum OPERATION { ADD,SUB,MUL,DIV }

     static BufferedImage imread(String path){
        try{
            BufferedImage  image;
            image = ImageIO.read(new File(path));           
            return image;
        }
        catch (IOException e){
           System.out.println("Error:"+e.getMessage()); 
           return null;
        }
    }
    
    static void imshow(BufferedImage im){
          
        try {
            ImageRenderComponent irc = new ImageRenderComponent(im);
            irc.setOpaque(true);  
            JFrame f = new JFrame("Display Image");
            f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);       
            f.setContentPane(irc);
            f.setSize(800,600);           
            f.setVisible(true);
        }
        catch (Exception e){
            e.printStackTrace();
        }
    }
    
    /*
     * the image formats supported by JRE are 
     * BMP, bmp
       jpg, JPG
       wbm
       jpeg
       png, PNG
       JPEG
       WBMP
       GIF, gif 
     */
    static void imsave(BufferedImage im,String path){
        try {         
            File outputfile = new File(path);
            String format = path.substring(path.lastIndexOf(".")+1, path.length());
            ImageIO.write(im, format, outputfile);
        } 
        catch (IOException e) {
            System.out.println("Error:"+e.getMessage()); 
        }
    }
    
    static BufferedImage imnew(int width, int height, String type){
        BufferedImage im=null;
        
        if (type.contentEquals("RGB")){
            im = new BufferedImage(width, height, BufferedImage.TYPE_3BYTE_BGR);
            for (int i=0;i<im.getHeight();i++){
                for (int j=0;j<im.getWidth();j++){
                    im.setRGB(j, i, 0xFF000000);
                }
            }
            
        }
        else if (type.contentEquals("GREY") || type.contentEquals("GRAY")){
            im = new BufferedImage(width, height, BufferedImage.TYPE_BYTE_GRAY);
        }
        
        return im;
    }

    static Kernel kernelinit(){
        Kernel k = new Kernel(3,3,new float[] {
                    0.0f, 0.0f, 0.0f,
                    0.0f, 1.0f, 0.0f,
                    0.0f, 0.0f, 0.0f
                });
        return k;
    }

    static Kernel kernelnew(int width, int height){

        try{
            float[] data = new float[width*height];
            Kernel k = new Kernel(width, height, data);
            return k;
        }
        catch (Exception e){
            e.printStackTrace();
            return null;
        }
    }

    static void print(Object data){        
        System.out.print(data);
    }
    
    
    static int getWidth(BufferedImage im){
        try {
            return im.getWidth();
        }
        catch (Exception e){
            e.printStackTrace();
            return -1;
        }                
    }
    
    static int getWidth(Kernel k){
        try {
            return k.getWidth();
        }
        catch (Exception e){
            e.printStackTrace();
            return -1;
        }    
    }
    
    static int getHeight(BufferedImage im){
        try {
            return im.getHeight();
        }
        catch (Exception e){
            e.printStackTrace();
            return -1;
        }    
    }
    
    static int getHeight(Kernel k){
        try {
            return k.getHeight();
        }
        catch (Exception e){
            e.printStackTrace();
            return -1;
        }    
    }
    
    static String getType(BufferedImage im){
        
        try{
            if (im.getType() == BufferedImage.TYPE_INT_ARGB || im.getType() == BufferedImage.TYPE_3BYTE_BGR || im.getType() == BufferedImage.TYPE_BYTE_INDEXED){
                return "RGB";
            }        
            else if (im.getType() == BufferedImage.TYPE_BYTE_GRAY) {
                return "GREY";
            }
            else {
                return "unknown type";
            }
        }
        catch (Exception e){
            e.printStackTrace();
            return "ERROR: getType()";
        }
    }
    
    static int getLength(String s){
        try {
            return s.length();
        }
        catch (Exception e){
            e.printStackTrace();
            return -1;
        }
    }

    static float getKernelData(Kernel k,int i, int j){
        float[] data = new float[k.getHeight()*k.getWidth()];
        k.getKernelData(data);
        return data[i+j*k.getWidth()];
    }

    static Kernel setKernelData(Kernel k, int i, int j, float value){
        float[] data = new float[k.getHeight()*k.getWidth()];
        k.getKernelData(data);
        data[i+j*k.getWidth()] = value;
        k = new Kernel(k.getWidth(),k.getHeight(),data);
        return k;
    }

   private static int objectToInt(Object o){
       if (o.getClass().getName().contentEquals("java.lang.Float")){
           return (int)Float.parseFloat(o.toString());
       }
       else if (o.getClass().getName().contentEquals("java.lang.Double")){
           return (int)Double.parseDouble(o.toString());
       }
       else if (o.getClass().getName().contentEquals("java.lang.Integer")){
           return Integer.parseInt(o.toString());
       }
       else if (o.getClass().getName().contentEquals("java.lang.String")){
           return Integer.parseInt(o.toString());
       }
       else {
           throw new RuntimeException("type error!");
       }
       
   }
   
   private static float objectToFloat(Object o){
       if (o.getClass().getName().contentEquals("java.lang.Float")){
           return Float.parseFloat(o.toString());
       }
       else if (o.getClass().getName().contentEquals("java.lang.Double")){
           return (float)Double.parseDouble(o.toString());
       }
       else if (o.getClass().getName().contentEquals("java.lang.Integer")){
           return (float)Integer.parseInt(o.toString());
       }
       else if (o.getClass().getName().contentEquals("java.lang.String")){
           return Float.parseFloat(o.toString());
       }
       else {
           throw new RuntimeException("type error!");
       }
   }
    static Object doArithmetic(Object op1, Object op2, OPERATION op){

            if (op1 instanceof BufferedImage){
                BufferedImage des;
                boolean color=false;
                try{
                    int t = ((BufferedImage)op1).getSampleModel().getSample(0, 0, 1, ((BufferedImage)op1).getRaster().getDataBuffer());
                    //des = imnew(((BufferedImage)op1).getWidth(),((BufferedImage)op1).getHeight(),"RGB");
                    des = new BufferedImage(((BufferedImage)op1).getWidth(),((BufferedImage)op1).getHeight(),((BufferedImage)op1).getType());
                    color=true;
                } catch (Exception e){
                    des = imnew(((BufferedImage)op1).getWidth(),((BufferedImage)op1).getHeight(),"GREY");
                }
                        
                if (op2 instanceof BufferedImage){
                    if (((BufferedImage)op1).getHeight()!=((BufferedImage)op2).getHeight() || ((BufferedImage)op1).getWidth()!=((BufferedImage)op2).getWidth())
                        throw new RuntimeException("dimension mismatch!");                                            
                }
                    
                       

                int v1;
                int v2;
                for (int j=0;j<((BufferedImage)des).getHeight();j++){
                    for (int i=0;i<((BufferedImage)des).getWidth();i++){
                        if (color){
                            v1 = ((BufferedImage)op1).getRGB(i, j);
                            if (op2 instanceof BufferedImage)
                                v2 = ((BufferedImage)op2).getRGB(i, j);
                            else 
                                v2 = objectToInt(op2);
                            
                                
                        }
                        else{
                            v1 = ((BufferedImage)op1).getSampleModel().getSample(i,j,0,((BufferedImage)op1).getRaster().getDataBuffer());
                            if (op2 instanceof BufferedImage)
                                v2 = ((BufferedImage)op2).getSampleModel().getSample(i,j,0,((BufferedImage)op2).getRaster().getDataBuffer());
                            else 
                                v2 = objectToInt(op2);
                        }

                        switch (op){
                            case ADD:
                                if (color){
                                    ((BufferedImage)des).setRGB(i, j, (int)v1+v2);
                                    break;
                                }
                                else{
                                    ((BufferedImage)des).getSampleModel().setSample(i, j, 0,(int)v1+v2 , ((BufferedImage)des).getRaster().getDataBuffer());
                                    break;
                                }
                            case SUB:
                               if (color){
                                    ((BufferedImage)des).setRGB(i, j, (int)v1-v2);                                    
                                    break;
                                }
                                else{
                                    ((BufferedImage)des).getSampleModel().setSample(i, j, 0,(int)v1-v2 , ((BufferedImage)des).getRaster().getDataBuffer());
                                    break;
                                }
                            case MUL:
                                if (color){
                                    ((BufferedImage)des).setRGB(i, j, (int)v1*v2);
                                    break;
                                }
                                else{
                                    ((BufferedImage)des).getSampleModel().setSample(i, j, 0,(int)v1*v2 , ((BufferedImage)des).getRaster().getDataBuffer());
                                    break;
                                }
                            case DIV:
                               if (color){
                                    ((BufferedImage)des).setRGB(i, j, (int)v1/v2);
                                    break;
                                }
                                else{
                                    ((BufferedImage)des).getSampleModel().setSample(i, j, 0,(int)v1/v2 , ((BufferedImage)des).getRaster().getDataBuffer());
                                    break;
                                }
                        }
//                        if (!((BufferedImage)des).isAlphaPremultiplied())
//                        ((BufferedImage)des).getSampleModel().setSample(i, j, 3,0xFF , ((BufferedImage)des).getRaster().getDataBuffer());
                    }
                } // end of for 
                return des;                
            }      
            
            else if (op1 instanceof Kernel){
                float[] des = new float[((Kernel)op1).getWidth()*((Kernel)op1).getHeight()];
                float[] data1 = new float[((Kernel)op1).getWidth()*((Kernel)op1).getHeight()];
                ((Kernel)op1).getKernelData(data1);
                
                float[] data2 = new float[((Kernel)op1).getWidth()*((Kernel)op1).getHeight()];
                
                if (op2 instanceof Kernel){
                    if (((Kernel)op1).getHeight()!=((Kernel)op2).getHeight() || ((Kernel)op1).getWidth()!=((Kernel)op2).getWidth())
                        throw new RuntimeException("dimension mismatch!");                                            
                    
                    
                    ((Kernel)op2).getKernelData(data2);
                }
                
                
                float v1;
                float v2;
                
                for (int i=0;i<des.length;i++){
                    v1 = data1[i];
                    if (op2 instanceof Kernel)
                        v2 = data2[i];
                    else 
                        v2 = objectToFloat(op2);
                    
                      switch (op){
                            case ADD:
                               des[i] = v1+v2;
                               break;
                            case SUB:
                                des[i] = v1-v2;
                                break;
                            case MUL:
                                  des[i] = v1*v2;
                                  break;
                            case DIV:
                              des[i] = v1/v2;
                               break;
                        }
                }
              
                return new Kernel(((Kernel)op1).getWidth(),((Kernel)op1).getHeight(),des);
            }
            else{
                throw new RuntimeException("op1 type error!");        
            }
    }

    static Object clone(Object src){
        if (src instanceof BufferedImage){
             String[] pnames = ((BufferedImage)src).getPropertyNames();
             Hashtable<String, Object> cproperties = new Hashtable<String,Object>();
             if(pnames != null) {
               for(int i = 0; i < pnames.length; i++) {
                   cproperties.put(pnames[i], ((BufferedImage)src).getProperty(pnames[i]));
               }
            }
            WritableRaster wr = ((BufferedImage)src).getRaster();
            WritableRaster cwr = wr.createCompatibleWritableRaster();
            cwr.setRect(wr);
           
            BufferedImage des = new BufferedImage(
               ((BufferedImage)src).getColorModel(),  // should be immutable
               cwr,
               ((BufferedImage)src).isAlphaPremultiplied(),
               cproperties);

            return des;
            
           
        }
        else if (src instanceof Kernel){
            return ((Kernel)src).clone();
        }
        else{
            throw new RuntimeException("the type of source is not cloneable!");
        }
    }

    static String read(){
        try{
            return (new BufferedReader(new InputStreamReader(System.in))).readLine();
        }
        catch(Exception e){
            e.printStackTrace();
            return null;
        }
    }

    static void setImagePixel(BufferedImage im, int col, int row, String channel, int value){
        try{
            if (channel.toUpperCase().contentEquals("RGB")){
                    im.setRGB(col, row, value|0xFF000000);
            }
            else if (channel.toUpperCase().contentEquals("R")){
                    im.getSampleModel().setSample(col, row, 0, value, im.getRaster().getDataBuffer());
            }
            else if (channel.toUpperCase().contentEquals("G")){
                    im.getSampleModel().setSample(col, row, 1, value, im.getRaster().getDataBuffer());                    
            }
            else if (channel.toUpperCase().contentEquals("B")){
                    im.getSampleModel().setSample(col, row, 2, value, im.getRaster().getDataBuffer());                   
            }
            else if (channel.toUpperCase().contentEquals("GREY") || channel.toUpperCase().contentEquals("GRAY")){
                    im.getSampleModel().setSample(col, row, 0, value, im.getRaster().getDataBuffer());
            }
            else{
                throw new RuntimeException("invalid channnel!");
            }
        }
        catch (Exception e){
            e.printStackTrace();
        }
    }

    static int getImagePixel(BufferedImage im, int col, int row, String channel){
        try{
            if (channel.toUpperCase().contentEquals("RGB")){
                return im.getRGB(col, row)&0x00FFFFFF;
            }
            else if (channel.toUpperCase().contentEquals("R")){
                    return im.getSampleModel().getSample(col, row, 0, im.getRaster().getDataBuffer());
            }
            else if (channel.toUpperCase().contentEquals("G")){
                    return im.getSampleModel().getSample(col, row, 1, im.getRaster().getDataBuffer());
            }
            else if (channel.toUpperCase().contentEquals("B")){
                    return im.getSampleModel().getSample(col, row, 2, im.getRaster().getDataBuffer());
            }
            else if (channel.toUpperCase().contentEquals("GREY") || channel.toUpperCase().contentEquals("GRAY")){
                    return im.getSampleModel().getSample(col, row, 0, im.getRaster().getDataBuffer());
            }
            else{
                throw new RuntimeException("invalid channnel!");
            }
        }
        catch(Exception e){
            e.printStackTrace();
            return -1;
        }
    }

    static BufferedImage convolve(BufferedImage im, Kernel k){
        BufferedImage imc = new BufferedImage(im.getWidth(), im.getHeight(), im.getType());
        ConvolveOp convo = new ConvolveOp(k);
        convo.filter(im, imc);
        return imc;
    }
    public static void main(String[] args) {

           Kernel k = new Kernel(3,3,new float[] {
                            0.0f, -1.0f, 0.0f,
                            -1.0f, 5.0f, -1.0f,
                            0.0f,  -1.0f, 0.0f
                        });

        //TYPE_3BYTE_BGR   BMP JPG
        //TYPE_BYTE_INDEXED  GIF
        BufferedImage im = imread("/home/hendry/Documents/PLT/project/matlip/rabbit.jpg");
       // imsave(im,"/home/hendry/Documents/PLT/project/matlip/tmp.bmp");
       // im = imread("/home/hendry/Documents/PLT/project/matlip/tmp.bmp");
       im = (BufferedImage)doArithmetic(im,1,OPERATION.ADD);
       imsave(im,"/home/hendry/Documents/PLT/project/matlip/rabbit5.jpg");
        imshow(im);
        BufferedImage mulim = (BufferedImage)doArithmetic(im,0,OPERATION.MUL);
        int a = getImagePixel(mulim,0,0,"RGB");
        System.out.println(a);
       // BufferedImage imm = convolve(im,k);
        //System.out.println(im.getType());
        //System.out.println(im.TYPE_3BYTE_BGR);
        //System.out.println(im.TYPE_BYTE_INDEXED);
        
        //imshow(imm);
        //System.out.println(getImagePixel(im,0,0,"G"));
       
       
       // imshow(mulim);

//        BufferedImage im_grey = (new ColorConvertOp(ColorSpace.getInstance(ColorSpace.CS_GRAY),null)).filter(im, null);
//        imshow(im_grey);
//        BufferedImage ro = imnew(im.getHeight(),im.getWidth(),"RGB");
//
//        for (int j=0;j<im.getHeight();j++){
//            for (int i=0;i<im.getWidth();i++){
//                ro.setRGB(j, i, im.getRGB(i, j));
//            }
//        }
      //  imshow(ro);
       


//        String s = "11";
//
//        int ii = objectToInt(s);
//        float ff = objectToFloat(ii);
//
//        System.out.println(ff);
//        System.out.println(ii);
//        float aa = 1.1f%2.2f;
//
//        String in = read();
//        System.out.println(in);
//        int a =10;
//        float b = 10f;
        //float aaaa = Math.sqrt(b);



//        BufferedImage imc = (BufferedImage)clone(im);
     //   imshow(im);
       // imshow(imc);
        //im = imnew(100,100,"RGB");
        //imshow(im);
        //imshow(imc);
        //BufferedImage sharpen = new ConvolveOp(k,ConvolveOp.EDGE_NO_OP,null).filter(im, null);
        //imshow(sharpen);
 
//
//        Kernel k = new Kernel(3,3,new float[] {
//                    0.0f, -1.0f, 0.0f,
//                    -1.0f, 5.0f, -1.0f,
//                    0.0f, -1.0f, 0.0f
//                });
//         Kernel k1 = new Kernel(3,3,new float[] {
//                    0.0f, -1.0f, 0.0f,
//                    -1.0f, 5.0f, -1.0f,
//                    0.0f, -1.0f, 0.0f
//                });
        //Kernel kk = new Kernel(3,3,new float[] {});
        //Kernel k1 = k+3;
           
//        Kernel k2 = (Kernel)doArithmetic(k,k1,OPERATION.SUB);
//        float[] data = new float[k.getHeight()*k.getWidth()];
//        k2.getKernelData(data);
//        
//        for (int i=0;i<data.length;i++)
//            System.out.print(data[i]+", ");
        
        
      
        //imshow(im);
        
        // kernel for sharpen        
//        Kernel k = new Kernel(3,3,new float[] {
//                    0.0f, -1.0f, 0.0f,
//                    -1.0f, 5.0f, -1.0f,
//                    0.0f, -1.0f, 0.0f
//                });

       
       
       /**
        * syntax for declare kernel type
        * Kernel K = new Kernel(x,y,new float[]{v1,v2,....});
        */
//        Kernel k1 = new Kernel(3,3,new float[] {
//                    1.0f/9.0f, 1.0f/9.0f, 1.0f/9.0f,
//                    1.0f/9.0f, 1.0f/9.0f, 1.0f/9.0f,
//                    1.0f/9.0f, 1.0f/9.0f, 1.0f/9.0f
//                });
//        
        
//        BufferedImage sharpen = new ConvolveOp(k,ConvolveOp.EDGE_NO_OP,null).filter(im, null);
//        imshow(sharpen);
//        
        /**
         * syntax for apply a kernel to an image
         * BufferedImage newIm = new ConvolveOp(k1,ConvolveOp.EDGE_NO_OP,null).filter(im, null);
         * im: original image
         * k1: kernel
         * newIm: new image
         */
//        BufferedImage blur = new ConvolveOp(k1,ConvolveOp.EDGE_NO_OP,null).filter(im, null);
//        imshow(blur);
        
      
      

    }

}


class ImageRenderComponent extends JPanel {
    BufferedImage image;
    Dimension size;
 
    public ImageRenderComponent(BufferedImage image) {
        this.image = image;
        size = new Dimension(image.getWidth(), image.getHeight());
    }
 
    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        int x = (getWidth() - size.width)/2;
        int y = (getHeight() - size.height)/2;
        g.drawImage(image, x, y, this);
    }
 
    @Override
    public Dimension getPreferredSize() {
        return size;
    }
}

