type op = Add | Sub | Mult | Div | Equal | Neq | Less | Leq | Greater | Geq | And | Or | Convolve | Power

type expr =
    Literal of int
  | Floatlit of float    
  | Id of string
  | Datatype of string
  | String of string
  | Binop of expr * op * expr
  | Assign of string * expr
  | AssignPixel of string * expr * expr * expr * expr
  | AssignKernel of string * expr * expr * expr      
  | Call of string * expr list
  | Uminus of expr
  | Not of expr  
  | ImageAccess of string * expr * expr * expr
  | KernelAccess of string * expr * expr
  | Imnew of expr * expr * string  
  | Kernelnew of expr * expr
  | Bool of string
    
type for_expr = 
    Assigna of string * expr * expr
  | Assignb of string * expr * expr * expr

type stmt =
    Block of stmt list
  | Expr of expr
  | KernelInit of string * expr list list
  | If of expr * stmt * stmt
  | Ifelseif of expr * stmt * elseif list * stmt
  | For of for_expr * stmt
  | While of expr * stmt
and elseif = {
    elseif_expr : expr;
    elseif_stmt : stmt;
  }

type var_decl = {
    varname : string;
    vartype : string;
  }

type func_decl = {
    fname : string;
    rettype : string;
    retname : string;
    formals : var_decl list;
    locals : var_decl list;
    body : stmt list;
  }

type program = var_decl list * func_decl list
