{ 
    open Parser 
    
    let incr_lineno lexbuf =
        let pos = lexbuf.Lexing.lex_curr_p in
            lexbuf.Lexing.lex_curr_p <- { pos with
                Lexing.pos_lnum = pos.Lexing.pos_lnum + 1;
                Lexing.pos_bol = pos.Lexing.pos_cnum;
            }
}

rule token = parse
  [' ' '\t'] { token lexbuf }
| ['\r' '\n']  { incr_lineno lexbuf; token lexbuf }
| '#'      { comment lexbuf }
| '"'      { string_type "" lexbuf}
| '('      { LPAREN }
| ')'      { RPAREN }
| '{'      { LBRACE }
| '}'      { RBRACE }
| '['      { LBRACKET }
| ']'      { RBRACKET }
| '@'      { CONVOLVE }
| '^'      { POWER }
| ';'      { SEMI }
| ','      { COMMA }
| ':'      { COLON }
| '+'      { PLUS }
| '-'      { MINUS }
| '*'      { TIMES }
| '/'      { DIVIDE }
| '='      { ASSIGN }
| "=="     { EQ }
| "!="     { NEQ }
| '<'      { LT }
| "<="     { LEQ }
| ">"      { GT }
| ">="     { GEQ }
| "and"    { AND }
| "or"     { OR }
| "not"    { NOT }
| "if"     { IF }
| "else"   { ELSE }
| "elseif" { ELSEIF }
| "for"    { FOR }
| "while"  { WHILE }
| "int"  { DATATYPE("int") }
| "float"  { DATATYPE("float") }
| "string" { DATATYPE("string") }
| "image" { DATATYPE("image") }
| "imnew" { NEWIMAGE }
| "kernel" { DATATYPE("kernel") }
| "kernelnew" { NEWKERNEL }
| "boolean"  { DATATYPE("boolean") }
| "end"    { END }
| "function" { FUNCTION }
| "true"  as lxm { BOOL(lxm) }
| "false"  as lxm { BOOL(lxm) }
| ['0'-'9']+ as lxm { LITERAL(int_of_string lxm) }
| ['0'-'9']+'.'['0'-'9']+(['E''e']['+''-']?['0'-'9']+)? as lxm { FLOATLIT(float_of_string lxm) }
| ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { ID(lxm) }
| eof { EOF }
| _ as char { 
            let pos = lexbuf.Lexing.lex_curr_p in
            raise (Failure("Illegal character: " ^ Char.escaped char
            ^ " in line #" ^ (string_of_int pos.Lexing.pos_lnum))) }

and comment = parse
  "\n" { incr_lineno lexbuf; token lexbuf }
| _    { comment lexbuf }

and string_type str = parse
   '"' { if (String.length str) > 0 then
            if str.[(String.length str)-1] = '\\' then
                string_type (str^(Lexing.lexeme lexbuf)) lexbuf
            else STRING(str)
         else STRING(str) }
|  "\n" {   let pos = lexbuf.Lexing.lex_curr_p in
            raise (Failure("Unclosed string literal, found beginning of" 
            ^ " a new line without closure of string; "
            ^ " in line #" ^ (string_of_int pos.Lexing.pos_lnum))) }
|  _ { string_type (str^(Lexing.lexeme lexbuf)) lexbuf }

